;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yoshi Coin, by mikeyk
;;
;; Description: When Mario collects this sprite, he gets a Hammer for a limited period 
;; of time -- similiar to that in Donkey Kong and Super Smash Bros.
;; 
;; Uses first extra bit: YES
;; If the first extra bit is set, Mario gets an increasing number of points with
;; each enemy he kills.  If he gets 8 or more, he starts collecting 1Ups
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; My RAM addresses
	RAM_ExtraBits = $7FAB10

	;; Original subroutines
	GivePoints = $02ACE5
	SprGfxRt1x2 = $019D5F

	;; Original RAM addresses
	RAM_SprOAMIndex = $15EA
	OAM_Tile	= $0302
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "INIT"
	LDA #$01
	STA $157C,x
	LDA RAM_ExtraBits,x
	AND #$04
	BEQ DoneInit
	LDA #$01
	STA $1632,x
DoneInit:	
        RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "MAIN"
	PHB
        PHK
        PLB
	JSR MainSub
	PLB
        RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Mario does not have hammer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
MainSub:
	JSR SubOffScreen

	LDA $64			; Call gfx routine with layer priority worked in
	PHA    
	LDA $1632,x
	BEQ DoGfx           
	LDA #$10                
	STA $64
DoGfx:	
	JSL SprGfxRt1x2	
	PLA
	STA $64
	
	LDY RAM_SprOAMIndex,X	; Overwrite tiles
	LDA #$60
	STA OAM_Tile,Y
	LDA #$62
	STA OAM_Tile+4,Y
	
	JSL $01A7DC		; Return if Mario isn't touching 
	BCC NoContact

	LDA #$1C		; Sound effect
	STA $1DF9

	INC $1422		; Increase Yoshi coin count
	INC $1420
	
	LDA $1420		; Give points
	CLC
	ADC #$03
	CMP #$08
	BCC SetPoints
	LDA #$08
SetPoints:
	JSL GivePoints
	JSR DisplayGlitter
	
	STZ $14C8,x		; Erase sprite
	
NoContact:	
	RTS



DisplayGlitter:
        LDY.B #$03		; Find free sprite effect slot 
LoopStart:
        LDA.W $17C0,Y		
        BEQ CreateGlitter	
        DEY                     
        BPL LoopStart           
Return:
	RTS			

CreateGlitter:	
	LDA #$05                
        STA $17C0,Y             

        LDA $E4,x                  
        STA $17C8,Y             
	LDA $D8,x
	CLC
	ADC #$08
        STA $17C4,Y             
        
        LDA #$10                
        STA $17CC,Y             
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffScreen:       JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry 
                    CLC                     
                    ADC TABLE4,y            
                    ROL $00                 
                    CMP $E4,x               ;x pos 
                    PHP                     
                    LDA $1B                 ;x boundry hi 
                    LSR $00                 
                    ADC TABLE5,y            
                    PLP                     
                    SBC $14E0,x             ;x pos high 
                    STA $00                 
                    LSR $01                 
                    BCC LABEL20             
                    EOR #$80                
                    STA $00                 
LABEL20             LDA $00                 
                    BPL RETURN_2            
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             
                    CPY #$FF                
                    BEQ KILL_SPRITE         
                    LDA #$00                ; \ mark sprite to come back   
                    STA $1938,y             ; /     
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                
                    STA $01                 
                    TAY                     
                    LDA $1C                 
                    CLC                     
                    ADC TABLE3,y            
                    ROL $00                 
                    CMP $D8,x               
                    PHP                     
                    LDA.W $001D             
                    LSR $00                 
                    ADC TABLE6,y            
                    PLP                     
                    SBC $14D4,x             
                    STA $00                 
                    LDY $01                 
                    BEQ LABEL22             
                    EOR #$80                
                    STA $00                 
LABEL22             LDA $00                 
                    BPL RETURN_2            
                    BMI ERASE_SPRITE        

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return
